"use strict";
const command_1 = require("@oclif/command");
const aws_sdk_1 = require("aws-sdk");
const chalk_1 = require("chalk");
class LambdaFunctionSearch extends command_1.Command {
    constructor() {
        super(...arguments);
        this.NextMarker = '';
        this.amount = 0;
    }
    async listAllFunctions(query = {}, functions = []) {
        if (!this.client) {
            throw new Error('Failed to initilize AWS SDK Class');
        }
        const params = {};
        if (this.NextMarker)
            params.Marker = this.NextMarker;
        const { NextMarker, Functions } = await this.client.listFunctions(params).promise();
        this.NextMarker = NextMarker || '';
        const targetFunctions = !Functions ? [] : Functions.filter(func => {
            if (query.Runtime)
                return func.Runtime === query.Runtime;
            return true;
        });
        const items = functions.concat(targetFunctions);
        this.amount += Functions ? Functions.length : 0;
        if (this.NextMarker) {
            return this.listAllFunctions(query, items);
        }
        return items;
    }
    async run() {
        const { flags } = this.parse(LambdaFunctionSearch);
        const props = {};
        if (!flags.region)
            this.log(chalk_1.default.yellow('warning') + ': Missing region');
        if (flags.region)
            props.region = flags.region;
        this.client = new aws_sdk_1.Lambda(props);
        const query = {};
        if (flags.runtime) {
            this.log(`${chalk_1.default.green('Search condition')}: Runtime === ${flags.runtime}`);
            query.Runtime = flags.runtime;
        }
        try {
            const result = await this.listAllFunctions(query);
            this.log(`${chalk_1.default.green('Matched Functions')}: ${result.length} / ${this.amount}`);
            this.log('====');
            result.forEach(item => {
                this.log(item.FunctionName);
                if (flags.showAll)
                    console.log(item);
            });
        }
        catch (e) {
            this.error(chalk_1.default.red(e));
            this.exit(1);
        }
    }
}
LambdaFunctionSearch.description = 'describe the command here';
LambdaFunctionSearch.flags = {
    help: command_1.flags.help({ char: 'h' }),
    // runtime
    runtime: command_1.flags.string({ char: 'R', description: 'Lambda runtime' }),
    // [For AWS SDK] region
    region: command_1.flags.string({
        char: 'r',
        description: 'region'
    }),
    // [For AWS SDK] profile
    profile: command_1.flags.string({
        char: 'p',
        description: 'AWS CLI profile'
    }),
    showAll: command_1.flags.boolean({
        char: 'A',
        description: 'Show all function data',
        default: false
    }),
};
module.exports = LambdaFunctionSearch;
